package com.dskj.znzn.transData.common.conf;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import springfox.bean.validators.configuration.BeanValidatorPluginsConfiguration;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.ParameterBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.schema.ModelRef;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Contact;
import springfox.documentation.service.Parameter;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2WebMvc;

import java.util.ArrayList;
import java.util.List;


@Configuration
@Import(BeanValidatorPluginsConfiguration.class)
@EnableSwagger2WebMvc
public class SwaggerConfiguration {

    @Value("${spring.profiles.active}")
    private String env;

    @Bean(value = "defaultApi")
    public Docket defaultApi() {
        List<Parameter> list = new ArrayList<>();
        ParameterBuilder headToken = new ParameterBuilder();
        headToken.name("token").description("TOKEN")
                .modelRef(new ModelRef("string")).parameterType("header").required(false)
                .defaultValue("")
                .build();

        list.add(headToken.build());

        return new Docket(DocumentationType.SWAGGER_2)
                .globalOperationParameters(list)
                .apiInfo(apiInfo())
                .groupName("dev".equals(env) ? "测试" : "生产")
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.dskj.znzn"))
                .paths(PathSelectors.any())
                .build();
    }

    private ApiInfo apiInfo() {
        return new ApiInfoBuilder()
                .title("中能智能数据转化项目接口文档")
                .termsOfServiceUrl("https://doc.weizhiliang.top/")
                .contact(new Contact("魏志亮", "https://www.weizhiliang.top", "727204053@qq.com"))
                .version("1.0")
                .build();
    }
}