package com.dskj.znzn.transData.common.filter;

import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

/**
 * 日志过滤器，用于记录日志
 *
 * @author 魏志亮
 */
@Slf4j
@Component
public class AccessLogFilter implements Filter {

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) throws IOException, ServletException {

        HttpServletRequest request = (HttpServletRequest) servletRequest;
        String path = request.getServletPath();

        List<String> urls = Arrays.asList(".", "api-docs");
        for (String url : urls) {
            if (path.contains(url)) {
                filterChain.doFilter(servletRequest, servletResponse);
                return;
            }
        }
        //如果上传文件,直接调用 doFilter
        if (null != request.getContentType() && request.getContentType().contains(MediaType.MULTIPART_FORM_DATA_VALUE)) {
            filterChain.doFilter(servletRequest, servletResponse);
            return;
        }

        filterChain.doFilter(new ParamsRequestWrapper(request), servletResponse);
    }

}
