package com.dskj.znzn.transData.web.dataTransfer.controller;

import com.dskj.znzn.transData.common.base.PageR;
import com.dskj.znzn.transData.common.base.R;
import com.dskj.znzn.transData.web.dataTransfer.entity.DataTransfer;
import com.dskj.znzn.transData.web.dataTransfer.inData.AddDataTransferTaskInData;
import com.dskj.znzn.transData.web.dataTransfer.inData.EditDataTransferTaskInData;
import com.dskj.znzn.transData.web.dataTransfer.inData.GetDataTransferInData;
import com.dskj.znzn.transData.web.dataTransfer.inData.GetTimeRangeInData;
import com.dskj.znzn.transData.web.dataTransfer.outData.ClassificationImportDataDetailsOutData;
import com.dskj.znzn.transData.web.dataTransfer.outData.GetStatisticalOutData;
import com.dskj.znzn.transData.web.dataTransfer.outData.GetTimeRangeOutData;
import com.dskj.znzn.transData.web.dataTransfer.outData.PltIndexMoreOutData;
import com.dskj.znzn.transData.web.dataTransfer.service.IDataTransferService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * <p>
 * 数据导入  前端控制器
 * </p>
 *
 * @author 魏志亮
 * @since 2025-01-09
 */
@RestController
@RequestMapping("/dataTransfer")
@Api(tags = {"数据导入"})
@RequiredArgsConstructor(onConstructor = @__({@Autowired}))
public class DataTransferController {

    private final IDataTransferService dataTransferService;

    @ApiOperation("获取数据导入记录")
    @PostMapping("/getDataTransfer")
    public PageR<DataTransfer> getDataTransfer(@RequestBody GetDataTransferInData data) {
        return dataTransferService.getDataTransfer(data);
    }

    @ApiOperation("根据ID查询任务")
    @GetMapping("/getById/{id}")
    public R<DataTransfer> getById(@Valid @NotNull @PathVariable Integer id) {
        return R.ok(dataTransferService.getById(id));
    }

    @ApiOperation("添加数据导入任务")
    @PostMapping("/addDataTransferTask")
    public R addDataTransferTask(@Valid @RequestBody AddDataTransferTaskInData data) {
        dataTransferService.addDataTransferTask(data);
        return R.ok();
    }

    @ApiOperation("编辑数据导入任务")
    @PostMapping("/editDataTransferTask")
    public R editDataTransferTask(@Valid @RequestBody EditDataTransferTaskInData data) {
        dataTransferService.editDataTransferTask(data);
        return R.ok();
    }

    @ApiOperation("删除数据导入任务")
    @GetMapping("/deleteDataTransferTask/{id}")
    public R deleteDataTransferTask(@Valid @NotNull @PathVariable Integer id) {
        dataTransferService.deleteDataTransferTask(id);
        return R.ok();
    }

    @ApiOperation("重新导入")
    @GetMapping("/reTransfer/{id}")
    public R reTransfer(@Valid @NotNull @PathVariable Integer id) {
        dataTransferService.reTransfer(id);
        return R.ok();
    }

    @ApiOperation("获取时间范围")
    @PostMapping("/getTimeRange")
    public R<List<GetTimeRangeOutData>> getTimeRange(@Valid @RequestBody GetTimeRangeInData data) {
        return R.ok(dataTransferService.getTimeRange(data));
    }

//    @ApiOperation("获取时间范围-在线数据")
//    @PostMapping("/getOnlineTimeRange")
//    public R<GetTimeRangeOutData> getOnlineTimeRange(@Valid @RequestBody GetOnlineTimeRangeInData data) {
//        return R.ok(dataTransferService.getOnlineTimeRange(data));
//    }


    @ApiOperation("获取统计信息")
    @PostMapping("/getStatisticalData")
    public R<GetStatisticalOutData> getStatisticalData() {
        return R.ok(dataTransferService.getStatisticalData());
    }

    @ApiOperation("最近导入成功的数据")
    @PostMapping("/recentlyImportedSuccessfulData")
    public R<DataTransfer> recentlyImportedSuccessfulData() {
        return R.ok(dataTransferService.recentlyImportedSuccessfulData());
    }


    @ApiOperation("分类导入数据详情")
    @PostMapping("/classificationImportDataDetails")
    public R<List<ClassificationImportDataDetailsOutData>> classificationImportDataDetails() {
        return R.ok(dataTransferService.classificationImportDataDetails());
    }

    @ApiOperation("最近导入成功的5条数据")
    @PostMapping("/recentlyImportedSuccessful5Data")
    public R<List<DataTransfer>> recentlyImportedSuccessful5Data() {
        return R.ok(dataTransferService.recentlyImportedSuccessful5Data());
    }

    @ApiOperation("平台更多详情")
    @PostMapping("/pltIndexMore")
    public R<List<PltIndexMoreOutData>> pltIndexMore() {
        return R.ok(dataTransferService.pltIndexMore());
    }


    @ApiOperation("优先级-提高或放弃优先级")
    @GetMapping("/increaseOrGiveUpPriority/{id}")
    public R increaseOrGiveUpPriority(@Valid @NotNull @PathVariable Integer id) {
        dataTransferService.increaseOrGiveUpPriority(id);
        return R.ok();
    }

    @ApiOperation("优先级-现在高优先级任务数量")
    @GetMapping("/highPriorityCount")
    public R<Long> highPriorityCount() {
        return R.ok(dataTransferService.highPriorityCount());
    }

}
