package com.dskj.znzn.transData.web.dataTransfer.controller;

import com.dskj.znzn.transData.common.base.R;
import com.dskj.znzn.transData.web.dataTransfer.entity.WindFarmDayCount;
import com.dskj.znzn.transData.web.dataTransfer.inData.HistoryDataSummaryInData;
import com.dskj.znzn.transData.web.dataTransfer.outData.HistoryDataSummaryOutData;
import com.dskj.znzn.transData.web.dataTransfer.outData.PltAutoAnalysisTimeOutData;
import com.dskj.znzn.transData.web.dataTransfer.service.IWindFarmDayCountService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.util.List;

/**
 * <p>
 * 每日采集数据表 前端控制器
 * </p>
 *
 * @author 魏志亮
 * @since 2025-03-24
 */
@RestController
@RequestMapping("/windFarmDayCount")
@RequiredArgsConstructor(onConstructor = @__({@Autowired}))
@Api(tags = {"每日采集数据表"})
public class WindFarmDayCountController {

    private final IWindFarmDayCountService windFarmDayCountService;

    @ApiOperation("获取今天数据")
    @PostMapping("getTodayData")
    public R<List<WindFarmDayCount>> getTodayData() {
        return R.ok(windFarmDayCountService.getTodayData());
    }

    @ApiOperation("获取历史数据汇总")
    @PostMapping("getHistoryDataSummary")
    public R<List<HistoryDataSummaryOutData>> getHistoryDataSummary(@RequestBody HistoryDataSummaryInData inData) {
        return R.ok(windFarmDayCountService.getHistoryDataSummary(inData));
    }

    @ApiOperation("获取自动分析数据时间")
    @GetMapping("pltAutoAnalysisTime")
    public R<PltAutoAnalysisTimeOutData> pltAutoAnalysisTime(@Valid @NotBlank(message = "风场编号不能为空") String windFarmCode) {
        return R.ok(windFarmDayCountService.pltAutoAnalysisTime(windFarmCode));
    }

    @ApiOperation("首页-获取在线数据")
    @PostMapping("getIndexOnlineData")
    public R<List<HistoryDataSummaryOutData>> getIndexOnlineData() {
        return R.ok(windFarmDayCountService.getIndexOnlineData());
    }

}
