package com.dskj.znzn.transData.web.dataTransfer.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dskj.znzn.transData.common.base.PageR;
import com.dskj.znzn.transData.common.exception.BusinessException;
import com.dskj.znzn.transData.web.common.entity.SysConf;
import com.dskj.znzn.transData.web.common.service.ISysConfService;
import com.dskj.znzn.transData.web.dataTransfer.entity.DataTransfer;
import com.dskj.znzn.transData.web.dataTransfer.inData.AddDataTransferTaskInData;
import com.dskj.znzn.transData.web.dataTransfer.inData.EditDataTransferTaskInData;
import com.dskj.znzn.transData.web.dataTransfer.inData.GetDataTransferInData;
import com.dskj.znzn.transData.web.dataTransfer.inData.GetTimeRangeInData;
import com.dskj.znzn.transData.web.dataTransfer.mapper.DataTransferMapper;
import com.dskj.znzn.transData.web.dataTransfer.outData.*;
import com.dskj.znzn.transData.web.dataTransfer.service.IDataTransferService;
import com.dskj.znzn.transData.web.dataTransfer.service.IWindFarmDayCountService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 数据操作  服务实现类
 * </p>
 *
 * @author 魏志亮
 * @since 2025-01-09
 */
@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired}))
@Slf4j
public class DataTransferServiceImpl extends ServiceImpl<DataTransferMapper, DataTransfer> implements IDataTransferService {

    private final DataTransferMapper dataTransferMapper;

    private final ISysConfService sysConfService;

    private final IWindFarmDayCountService windFarmDayCountService;

    @Override
    public PageR<DataTransfer> getDataTransfer(GetDataTransferInData data) {
        IPage<DataTransfer> queryPage = Page.of(data.getPageNo(), data.getPageSize());
        LambdaQueryWrapper<DataTransfer> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(StringUtils.isNotBlank(data.getWindFarmCode()), DataTransfer::getWindFarmCode, data.getWindFarmCode());
        wrapper.eq(null != data.getTransferStatus(), DataTransfer::getTransferStatus, data.getTransferStatus());
        wrapper.eq(StringUtils.isNotBlank(data.getTransferType()), DataTransfer::getTransferType, data.getTransferType());
        wrapper.eq(DataTransfer::getStatus, 1);
        wrapper.orderByDesc(DataTransfer::getUpdateTime);
        IPage<DataTransfer> pageData = page(queryPage, wrapper);
        for (DataTransfer record : pageData.getRecords()) {
            record.setTransferTypeName(sysConfService.getValueByType("trans_type", record.getTransferType()));
        }

        return PageR.page(pageData, pageData.getRecords());
    }

    @Override
    public void addDataTransferTask(AddDataTransferTaskInData data) {
        DataTransfer dataTransfer = new DataTransfer();
        BeanUtils.copyProperties(data, dataTransfer);
        dataTransfer.setCreateTime(new Date());
        dataTransfer.setTransferStatus(-1);
        save(dataTransfer);
    }

    @Override
    public void editDataTransferTask(EditDataTransferTaskInData data) {

        DataTransfer query = getById(data.getId());
        if (query == null || query.getStatus() == 0) {
            throw new RuntimeException("数据不存在");
        }
        if (query.getTransferStatus() == 0 || query.getTransferStatus() == 1) {
            throw new RuntimeException("导入中或导入成功不能编辑");
        }

        DataTransfer dataTransfer = new DataTransfer();
        BeanUtils.copyProperties(data, dataTransfer);
        updateById(dataTransfer);
    }

    @Override
    public void reTransfer(Integer id) {
        DataTransfer dataTransfer = getById(id);
        if (dataTransfer == null || dataTransfer.getStatus() == 0) {
            throw new RuntimeException("数据不存在");
        }

        if (dataTransfer.getTransferStatus() != 2) {
            throw new RuntimeException("导入失败后才能重新导入");
        }
        dataTransfer = new DataTransfer();
        dataTransfer.setId(id);
        dataTransfer.setTransferStatus(-1);
        updateById(dataTransfer);
    }

    @Override
    public void deleteDataTransferTask(Integer id) {
        DataTransfer query = getById(id);
        if (query == null || query.getStatus() == 0) {
            throw new RuntimeException("数据不存在");
        }
        if (query.getTransferStatus() == 0 || query.getTransferStatus() == 1) {
            throw new RuntimeException("导入中或导入成功不能删除");
        }
        DataTransfer dataTransfer = new DataTransfer();
        dataTransfer.setId(id);
        dataTransfer.setStatus(0);
        updateById(dataTransfer);
    }

    @Override
    public List<GetTimeRangeOutData> getTimeRange(GetTimeRangeInData data) {
        List<GetTimeRangeOutData> results = new ArrayList<>();
        String isOlineData = sysConfService.getValueByType("system_online_data", "is_online_data");
        if (null == isOlineData || "0".equals(isOlineData)) {
            LambdaQueryWrapper<DataTransfer> wrapper = Wrappers.lambdaQuery();
            wrapper.eq(DataTransfer::getWindFarmCode, data.getWindFarmCode())
                    .in(DataTransfer::getTransferType, data.getTransferType())
                    .eq(DataTransfer::getTransferStatus, 1)
                    .eq(DataTransfer::getStatus, 1)
                    .isNotNull(DataTransfer::getDataMinTime)
                    .isNotNull(DataTransfer::getDataMaxTime);

            List<DataTransfer> datas = list(wrapper);
            if (datas.isEmpty()) {
                throw new BusinessException("未查询到已导入完成的数据");
            }


            for (DataTransfer dataTransfer : datas) {
                results.add(new GetTimeRangeOutData(dataTransfer.getDataMinTime(), dataTransfer.getDataMaxTime()));
            }
            return mergePeriods(results);
        } else {
            List<GetTimeRangeOutData> datas = windFarmDayCountService.getTimeRange(data);
            if (null == datas.get(0)) {
                return Collections.emptyList();
            }
            return datas;
        }
    }

    public static List<GetTimeRangeOutData> mergePeriods(List<GetTimeRangeOutData> periods) {
        if (periods.size() <= 1) {
            return periods;
        }

        // 按开始时间排序
        periods.sort(Comparator.comparing(GetTimeRangeOutData::getStartTime));

        List<GetTimeRangeOutData> merged = new ArrayList<>();
        GetTimeRangeOutData current = periods.get(0);

        for (int i = 1; i < periods.size(); i++) {
            GetTimeRangeOutData next = periods.get(i);
            // 连续数据可能时间差小一天则合并
            if (current.getEndTime().after(new Date(next.getStartTime().getTime() - 86400000))) {
                current.setEndTime(current.getEndTime().after(next.getEndTime()) ? current.getEndTime() : next.getEndTime());
            } else {
                // 无重叠，添加当前时间段到结果列表，并移动到下一个时间段
                merged.add(current);
                current = next;
            }
        }
        // 添加最后一个时间段
        merged.add(current);

        return merged;
    }


    @Override
    public GetStatisticalOutData getStatisticalData() {
        GetStatisticalOutData data = new GetStatisticalOutData();

        LambdaQueryWrapper<DataTransfer> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(DataTransfer::getStatus, 1).select(DataTransfer::getWindFarmCode, DataTransfer::getId, DataTransfer::getTransferStatus);
        List<DataTransfer> datas = list(wrapper);
        data.setWindFarmCount((int) datas.stream().map(DataTransfer::getWindFarmCode).distinct().count());

        int total = datas.size();
        int success = (int) datas.stream().filter(dataTransfer -> dataTransfer.getTransferStatus() == 1).count();
        int fail = (int) datas.stream().filter(dataTransfer -> dataTransfer.getTransferStatus() == 2).count();

        data.setTransSuccessCount(String.format("%s/%s", success, total));
        data.setTransFailCount(fail);
        return data;
    }

    @Override
    public DataTransfer recentlyImportedSuccessfulData() {
        LambdaQueryWrapper<DataTransfer> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(DataTransfer::getTransferStatus, 1).eq(DataTransfer::getStatus, 1);
        wrapper.orderByDesc(DataTransfer::getUpdateTime);
        wrapper.last("limit 1");
        DataTransfer transfer = getOne(wrapper);
        if (null != transfer) {
            transfer.setTransferTypeName(sysConfService.getValueByType("trans_type", transfer.getTransferType()));
        }
        return transfer;
    }

    @Override
    public List<ClassificationImportDataDetailsOutData> classificationImportDataDetails() {
        List<ClassificationImportDataDetailsQuery> datas = dataTransferMapper.classificationImportDataDetails();
        List<SysConf> confList = sysConfService.getByType("trans_type");
        Map<String, List<ClassificationImportDataDetailsQuery>> windFarmMap = new HashMap<>();

        for (ClassificationImportDataDetailsQuery query : datas) {
            if (windFarmMap.containsKey(query.getWindFarmCode())) {
                windFarmMap.get(query.getWindFarmCode()).add(query);
            } else {
                List<ClassificationImportDataDetailsQuery> list = new ArrayList<>();
                list.add(query);
                windFarmMap.put(query.getWindFarmCode(), list);
            }
        }

        List<ClassificationImportDataDetailsOutData> results = new ArrayList<>();
        windFarmMap.forEach((key, value) -> {
            Map<String, ClassificationImportDataDetailsQuery> typeMap = new HashMap<>();
            for (ClassificationImportDataDetailsQuery query : value) {
                typeMap.put(query.getTransferType(), query);
            }

            ClassificationImportDataDetailsOutData outData = new ClassificationImportDataDetailsOutData();
            outData.setWindFarmCode(key);
            outData.setWindFarmName(value.get(0).getWindFarmName());

            List<TypeCountData> typeCountDataList = new ArrayList<>();
            for (SysConf sysConf : confList) {
                TypeCountData data = new TypeCountData();
                if (typeMap.containsKey(sysConf.getParamKey())) {
                    data.setTransferType(sysConf.getParamKey());
                    data.setTransferName(sysConf.getParamValue());
                    data.setCount(typeMap.get(sysConf.getParamKey()).getCount());
                    data.setLatestStorageTime(typeMap.get(sysConf.getParamKey()).getLatestStorageTime());
                } else {
                    data.setTransferType(sysConf.getParamKey());
                    data.setTransferName(sysConf.getParamValue());
                    data.setCount(0);
                    data.setLatestStorageTime(null);
                }
                typeCountDataList.add(data);
            }
            outData.setTypeCountDataList(typeCountDataList);
            results.add(outData);
        });

        return results;
    }

    @Override
    public List<DataTransfer> recentlyImportedSuccessful5Data() {
        LambdaQueryWrapper<DataTransfer> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(DataTransfer::getTransferStatus, 1).eq(DataTransfer::getStatus, 1);
        wrapper.orderByDesc(DataTransfer::getUpdateTime);
        wrapper.last("limit 5");
        List<DataTransfer> transferList = list(wrapper);
        if (!transferList.isEmpty()) {
            for (DataTransfer transfer : transferList) {
                transfer.setTransferTypeName(sysConfService.getValueByType("trans_type", transfer.getTransferType()));
            }
        }
        return transferList;
    }

    @Override
    public List<PltIndexMoreOutData> pltIndexMore() {
        List<ClassificationImportDataDetailsQuery> datas = dataTransferMapper.classificationImportDataDetails();
        List<SysConf> confList = sysConfService.getByType("trans_type");
        Map<String, List<ClassificationImportDataDetailsQuery>> windFarmMap = new HashMap<>();

        for (ClassificationImportDataDetailsQuery query : datas) {
            if (windFarmMap.containsKey(query.getWindFarmCode())) {
                windFarmMap.get(query.getWindFarmCode()).add(query);
            } else {
                List<ClassificationImportDataDetailsQuery> list = new ArrayList<>();
                list.add(query);
                windFarmMap.put(query.getWindFarmCode(), list);
            }
        }
        List<PltIndexMoreOutData> pltIndexMoreOutDataList = new ArrayList<>();
        windFarmMap.forEach((key, value) -> {
            Map<String, ClassificationImportDataDetailsQuery> typeMap = new HashMap<>();
            for (ClassificationImportDataDetailsQuery query : value) {
                typeMap.put(query.getTransferType(), query);
            }

            PltIndexMoreOutData data = new PltIndexMoreOutData();
            data.setWindFarmCode(key);
            data.setWindFarmName(value.get(0).getWindFarmName());

            for (SysConf sysConf : confList) {
                ClassificationImportDataDetailsQuery query = typeMap.get(sysConf.getParamKey());
                String type = sysConf.getParamKey();
                switch (type) {
                    case "minute":
                        data.setMinuteTimeAreaList(converAreaStrToList(typeMap.containsKey(type), query));
                        break;
                    case "second":
                        data.setSecondTimeAreaList(converAreaStrToList(typeMap.containsKey(type), query));
                        break;
                    case "warn":
                        data.setWarnTimeAreaList(converAreaStrToList(typeMap.containsKey(type), query));
                        break;
                    case "fault":
                        data.setFaultTimeAreaList(converAreaStrToList(typeMap.containsKey(type), query));
                        break;
                    case "wave":
                        data.setVibrationTimeAreaList(converAreaStrToList(typeMap.containsKey(type), query));
                        break;
                    default:
                        log.warn("当前类型" + type + "未显示");

                }
            }
            pltIndexMoreOutDataList.add(data);
        });

        return pltIndexMoreOutDataList;
    }

    @Override
    public void increaseOrGiveUpPriority(Integer id) {
        DataTransfer dataTransfer = getById(id);
        if (dataTransfer == null || dataTransfer.getStatus() == 0) {
            throw new RuntimeException("数据不存在");
        }
        Integer isHighPriority = dataTransfer.getIsHighPriority();
        dataTransfer = new DataTransfer();
        dataTransfer.setId(id);
        dataTransfer.setIsHighPriority(isHighPriority == 0 ? 1 : 0);
        updateById(dataTransfer);
    }

    @Override
    public Long highPriorityCount() {
        LambdaQueryWrapper<DataTransfer> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(DataTransfer::getIsHighPriority, 1);
        return count(wrapper);
    }


    private List<String> converAreaStrToList(boolean contains, ClassificationImportDataDetailsQuery query) {
        if (null == query) {
            return Collections.emptyList();
        }
        String timeAreaStr = query.getTimeArea();
        if (null == timeAreaStr || StringUtils.isBlank(timeAreaStr) || !contains) {
            return Collections.emptyList();
        } else {
            List<String> areaList = Arrays.asList(timeAreaStr.split(","));
            Collections.sort(areaList);
            return areaList;
        }
    }


    public static void main(String[] args) throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<GetTimeRangeOutData> periods = new ArrayList<>();
        periods.add(new GetTimeRangeOutData(sdf.parse("2024-01-01 00:00:00"), sdf.parse("2024-03-31 23:59:59")));
        periods.add(new GetTimeRangeOutData(sdf.parse("2024-04-01 00:00:00"), sdf.parse("2024-05-31 23:59:59")));
        periods.add(new GetTimeRangeOutData(sdf.parse("2024-08-01 00:00:00"), sdf.parse("2024-12-31 23:59:59")));

        List<GetTimeRangeOutData> mergedPeriods = mergePeriods(periods);
        for (GetTimeRangeOutData period : mergedPeriods) {
            System.out.println(sdf.format(period.getStartTime()) + " " + sdf.format(period.getEndTime()));
        }
    }
}
