package com.dskj.znzn.transData.web.dataTransfer.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dskj.znzn.transData.common.exception.BusinessException;
import com.dskj.znzn.transData.web.common.service.ISysConfService;
import com.dskj.znzn.transData.web.dataTransfer.entity.WindFarmDayCount;
import com.dskj.znzn.transData.web.dataTransfer.inData.GetTimeRangeInData;
import com.dskj.znzn.transData.web.dataTransfer.inData.HistoryDataSummaryInData;
import com.dskj.znzn.transData.web.dataTransfer.mapper.WindFarmDayCountMapper;
import com.dskj.znzn.transData.web.dataTransfer.outData.GetTimeRangeOutData;
import com.dskj.znzn.transData.web.dataTransfer.outData.HistoryDataSummaryOutData;
import com.dskj.znzn.transData.web.dataTransfer.outData.PltAutoAnalysisTimeOutData;
import com.dskj.znzn.transData.web.dataTransfer.service.IWindFarmDayCountService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 每日采集数据表 服务实现类
 * </p>
 *
 * @author 魏志亮
 * @since 2025-02-28
 */
@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired}))
public class WindFarmDayCountServiceImpl extends ServiceImpl<WindFarmDayCountMapper, WindFarmDayCount> implements IWindFarmDayCountService {

    private final WindFarmDayCountMapper windFarmDayCountMapper;
    private final ISysConfService sysConfService;

    private final SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");

    @Override
    public List<GetTimeRangeOutData> getTimeRange(GetTimeRangeInData data) {
        return windFarmDayCountMapper.getTimeRange(data);
    }

    @Override
    public List<WindFarmDayCount> getTodayData() {
        LambdaQueryWrapper<WindFarmDayCount> queryWrapper = Wrappers.lambdaQuery(WindFarmDayCount.class);
        queryWrapper.eq(WindFarmDayCount::getAddDate, dateFormat.format(new Date()));
        queryWrapper.eq(WindFarmDayCount::getDelStatus, 0);
        queryWrapper.eq(WindFarmDayCount::getSyncStatus, 0);
        queryWrapper.orderByDesc(WindFarmDayCount::getWindFarmCode);
        List<WindFarmDayCount> datas = list(queryWrapper);
        for (WindFarmDayCount data : datas) {
            data.setTypeName(sysConfService.getValueByType("trans_type", data.getType()));
        }
        return datas;
    }

    @Override
    public List<HistoryDataSummaryOutData> getHistoryDataSummary(HistoryDataSummaryInData inData) {
        List<HistoryDataSummaryOutData> datas = windFarmDayCountMapper.getHistoryDataSummary(inData);
        for (HistoryDataSummaryOutData data : datas) {
            data.setTypeName(sysConfService.getValueByType("trans_type", data.getType()));
        }
        return datas;
    }

    @Override
    public PltAutoAnalysisTimeOutData pltAutoAnalysisTime(String windFarmCode) {
        PltAutoAnalysisTimeOutData data = windFarmDayCountMapper.platformAutoAnalysisTime(windFarmCode);
        if (null == data) {
            throw new BusinessException("当前风场无数据");
        }
        return data;
    }

    @Override
    public List<HistoryDataSummaryOutData> getIndexOnlineData() {
        return windFarmDayCountMapper.getIndexOnlineData();
    }
}
