
package com.dskj.znzn.transData.web.waveData.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dskj.znzn.transData.common.base.PageR;
import com.dskj.znzn.transData.common.exception.BusinessException;
import com.dskj.znzn.transData.web.waveData.entity.WaveData;
import com.dskj.znzn.transData.web.waveData.indata.GetMesureDataInData;
import com.dskj.znzn.transData.web.waveData.indata.GetMesureWithSFDataInData;
import com.dskj.znzn.transData.web.waveData.mapper.WaveDataMapper;
import com.dskj.znzn.transData.web.waveData.outdata.ItemOutData;
import com.dskj.znzn.transData.web.waveData.service.IWaveDataService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired}))
public class WaveDataServiceImpl extends ServiceImpl<WaveDataMapper, WaveData> implements IWaveDataService {
    private final WaveDataMapper waveDataMapper;

    public List<WaveData> getMesureData(GetMesureDataInData inData) {
        if (inData.getWindCode().contains(" ")) {
            throw new BusinessException("风场参数异常");
        } else {
            return this.waveDataMapper.getMesureData(inData.getWindCode() + "_wave",
                    inData.getWindTurbineNumberList(), inData.getStartTime(), inData.getEndTime(), inData.getMesureNameList());
        }
    }

    public WaveData getById(String windCode, Integer id) {
        if (windCode.contains(" ")) {
            throw new BusinessException("风场参数异常");
        } else {
            return this.waveDataMapper.getById(windCode + "_wave", id);
        }
    }

    @Override
    public PageR<WaveData> getMesureDataWithSF(GetMesureWithSFDataInData inData) {
        if (inData.getWindCode().contains(" ")) {
            throw new BusinessException("风场参数异常");
        } else {
            Page<WaveData> queryPage = Page.of(inData.getPageNo(), inData.getPageSize());
            inData.setWindCode(inData.getWindCode() + "_wave");
            Page<WaveData> pageData = this.waveDataMapper.getMesureDataWithSF(queryPage, inData);
            return PageR.page(pageData);
        }
    }

    @Override
    public List<String> getAllSamplingFrequency(String windCode) {

        if (windCode.contains(" ")) {
            throw new BusinessException("风场参数异常");
        } else {
            windCode = windCode + "_wave";
            return waveDataMapper.getAllSamplingFrequency(windCode);
        }
    }

    @Override
    public List<ItemOutData> getWindTurbineNumberMesurePointName(String windCode, String windTurbineNumber) {
        if (windCode.contains(" ")) {
            throw new BusinessException("风场参数异常");
        } else {
            windCode = windCode + "_wave";
            return waveDataMapper.getWindTurbineNumberMesurePointName(windCode, windTurbineNumber);
        }
    }

    @Override
    public List<String> getWindTurbineNumberSamplingFrequency(String windCode, String windTurbineNumber) {
        if (windCode.contains(" ")) {
            throw new BusinessException("风场参数异常");
        } else {
            windCode = windCode + "_wave";
            return waveDataMapper.getWindTurbineNumberSamplingFrequency(windCode, windTurbineNumber);
        }
    }
}
